@@ -6,7 +6,7 @@
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
 
-BEGIN { $| = 1; print "1..1\n"; }
+BEGIN { $| = 1; print "1..2\n"; }
 END {print "not ok 1\n" unless $loaded;}
 use OpenCA::OpenSSL;
 $loaded = 1;
@@ -18,15 +18,14 @@ print "ok 1\n";
 # (correspondingly "not ok 13") depending on the success of chunk 13
 # of the test code):
 my $openpath=`type -path openssl`;
-chomp $openpath;
 
 my $openssl = new OpenCA::OpenSSL( SHELL => $openpath);
 
-my $k = {
-	CONFIG => "/usr/ssl/openssl.cnf"
-	};
-
+my $k = { CONFIG => "/usr/ssl/openssl.cnf" };
 $openssl->setParams( $k );
 
-print "ok 13\n";
+print "ok 2\n";
+print "\nAll tests ok.\n\n";
+
+exit 0;
 
